//**********************************************************
//      Delphi
//  - 1.0.1
//  - GPL v.1
//  - TEvg
// **********************************************************

unit Diagram;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls;

type

  TDateTimeMode = set of (mYear, mMonth, mDay, mHour, mMin, mSec, mMSec);

  TDrawEvent = procedure(Sender: TObject) of object;

  TVariantPoint = record
    X: variant;
    Y: variant;
  end;

  TVariantMarker = record
    X: variant;
    Y: variant;
  end;

  TPPoint = ^TVariantPoint;
  TPMarker = ^TVariantMarker;

  TAxisType = (atAbsciss, atOrdinate);

  TCustomDiagram = class(TImage)
  private
    { Private declarations }
    FWidth: integer;
    FHeight: integer;
    function GetColorBackground: TColor; virtual; abstract;
    procedure SetColorBackground(Value: TColor); virtual; abstract;
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
    { Published declarations }
    property ColorBackground: TColor read GetColorBackground write SetColorBackground;
  end;

  TCustomCurve = class(TObject)
  private
    { Private declarations }
    function GetDiagram: TCustomDiagram; virtual; abstract;
    procedure SetDiagram(Value: TCustomDiagram); virtual; abstract;
    function GetMinX: variant; virtual; abstract;
    procedure SetMinX(Value: variant); virtual; abstract;
    function GetMinY: variant; virtual; abstract;
    procedure SetMinY(Value: variant); virtual; abstract;
    function GetMaxX: variant; virtual; abstract;
    procedure SetMaxX(Value: variant); virtual; abstract;
    function GetMaxY: variant; virtual; abstract;
    procedure SetMaxY(Value: variant); virtual; abstract;
    procedure SetVisible(Value: boolean); virtual; abstract;
    function GetVisible: boolean; virtual; abstract;
    property Diagram: TCustomDiagram read GetDiagram write SetDiagram;
  public
    { Public declarations }
    property MinX: variant read GetMinX write SetMinX;
    property MaxX: variant read GetMaxX write SetMaxX;
    property MinY: variant read GetMinY write SetMinY;
    property MaxY: variant read GetMaxY write SetMaxY;
    property Visible: boolean read GetVisible write SetVisible;
  end;

  TCustomAxis = class(TObject)
  private
    { Private declarations }
    function GetCurve: TCustomCurve; virtual; abstract;
    procedure SetCurve(Curve: TCustomCurve); virtual; abstract;
    property Curve: TCustomCurve read GetCurve write SetCurve;
  public
    { Public declarations }
  end;

  TCustomGrid = class(TObject)
  private
    { Private declarations }
    function GetCurve: TCustomCurve; virtual; abstract;
    procedure SetCurve(Curve: TCustomCurve); virtual; abstract;
    property Curve: TCustomCurve read GetCurve write SetCurve;
  public
    { Public declarations }
  end;

  TGrid = class(TCustomGrid)	// ,    
  private
    { Private declarations }
    FCurve: TCustomCurve;
    FGridStepX: variant;
    FGridStepY: variant;
    FVisible: boolean;
    FPen: TPen;
    function GetCurve: TCustomCurve; override;
    procedure SetCurve(lCurve: TCustomCurve); override;
    procedure SetGridStepX(Value: variant);
    procedure SetGridStepY(Value: variant);
    procedure SetVisible(Value: boolean);
  public
    { Public declarations }
    constructor Create(lCurve: TCustomCurve);
    destructor Destroy; override;
    procedure Draw;		                                    //   
    property Pen: TPen read FPen write FPen;                        //     
    property GridStepX: variant read FGridStepX write SetGridStepX; //     
    property GridStepY: variant read FGridStepY write SetGridStepY; //     
    property Visible: boolean read FVisible write SetVisible;	    //     
  end;

  TAxis = class(TCustomAxis)	// ,    
  private
    { Private declarations }
    FCoordinate: variant;
    FArrowWidth: integer;
    FArrowLeng: integer;
    FVisible: boolean;
    FVisibleArrow: boolean;
    FVisibleScale: boolean;
    FIndentMin: integer;
    FIndentMax: integer;
    FScaleStep: variant;
    FScalePointReference: variant;
    FAxisType: TAxisType;
    FDistance: integer;
    FScaleSize: integer;
    FScaleEmptyIntervalMin: variant;
    FScaleEmptyIntervalMax: variant;
    FDateTimeMode: TDateTimeMode;
    FFloatPrecision: byte;
    FFloatDigits: byte;
    FPen: TPen;
    FScalePen: TPen;
    FFont: TFont;
    FCurve: TCustomCurve;
    function ToStr(Value: variant): TStringList;
    function GetCurve: TCustomCurve; override;
    procedure SetCurve(lCurve: TCustomCurve); override;
    procedure SetCoordinate(Value: variant);
    procedure SetArrowWidth(Value: integer);
    procedure SetArrowLeng(Value: integer);
    procedure SetVisible(Value: boolean);
    procedure SetVisibleArrow(Value: boolean);
    procedure SetVisibleScale(Value: boolean);
    procedure SetIndentMin(Value: integer);
    procedure SetIndentMax(Value: integer);
    procedure SetScaleStep(Value: variant);
    procedure SetScalePointReference(Value: variant);
    procedure SetAxisType(Value: TAxisType);
    procedure SetDistance(Value: integer);
    procedure SetScaleSize(Value: integer);
    procedure SetScaleEmptyIntervalMin(Value: variant);
    procedure SetScaleEmptyIntervalMax(Value: variant);
    procedure DrawX;
    procedure DrawY;
  public
    { Public declarations }
    constructor Create(lCurve: TCustomCurve);
    destructor Destroy; override;
    procedure Draw;					//   
    property Pen: TPen read FPen write FPen;	//     
    property ScalePen: TPen read FScalePen write FScalePen; //    
    property Coordinate: variant read FCoordinate write SetCoordinate;	//   
    property ArrowWidth: integer read FArrowWidth write SetArrowWidth;	//    , 
    property ArrowLeng: integer read FArrowLeng write SetArrowLeng;	//    , 
    property Visible: boolean read FVisible write SetVisible;		//    
    property VisibleArrow: boolean read FVisibleArrow write SetVisibleArrow; //    
    property VisibleScale: boolean read FVisibleScale write SetVisibleScale; //    
    property IndentMin: integer read FIndentMin write SetIndentMin; //      , 
    property IndentMax: integer read FIndentMax write SetIndentMax; //       , 
    property ScaleStep: variant read FScaleStep write SetScaleStep; //  
    property ScalePointReference: variant read FScalePointReference //    ( )
      write SetScalePointReference;
    property AxisType: TAxisType read FAxisType write SetAxisType;  //    (  )
    property DateTimeMode: TDateTimeMode read FDateTimeMode write FDateTimeMode; //       
    property Distance: integer read FDistance write SetDistance;    //     , 
    property FloatPrecision: byte read FFloatPrecision write FFloatPrecision;	//      ..
    property FloatDigits: byte read FFloatDigits write FFloatDigits;	     	// ..   
    property ScaleSize: integer read FScaleSize write SetScaleSize;		//   , 
    property ScaleEmptyIntervalMin: variant
      read FScaleEmptyIntervalMin write SetScaleEmptyIntervalMin;		//      ..
    property ScaleEmptyIntervalMax: variant
      read FScaleEmptyIntervalMax write SetScaleEmptyIntervalMax;		//..    
    property Font: TFont read FFont write FFont;				// ,    
  end;

  TCurve = class(TCustomCurve) // ,   
  private
    { Private declarations }
    FMinX: variant;
    FMaxX: variant;
    FMinY: variant;
    FMaxY: variant;
    FDiagram: TCustomDiagram;
    FVisible: boolean;
    FGrid: TGrid;
    FAxisX: TAxis;
    FAxisY: TAxis;
    FPen: TPen;
    FPointList: TList;
    FMarkerList: TList;
    FID: string;
    FMarkerColor: TColor;
    function GetDiagram: TCustomDiagram; override;
    procedure SetDiagram(Value: TCustomDiagram); override;
    function GetMinX: variant; override;
    procedure SetMinX(Value: variant); override;
    function GetMinY: variant; override;
    procedure SetMinY(Value: variant); override;
    function GetMaxX: variant; override;
    procedure SetMaxX(Value: variant); override;
    function GetMaxY: variant; override;
    procedure SetMaxY(Value: variant); override;
    function GetVisible: boolean; override;
    procedure SetVisible(Value: boolean); override;
    function GetPoint(Index: integer): TVariantPoint;
    procedure SetPoint(Index: integer; Value: TVariantPoint);
    function GetMarker(Index: integer): TVariantMarker;
    procedure SetMarker(Index: integer; Value: TVariantMarker);
    function GetID: string;
    procedure SetID(Value: string);
  public
    { Public declarations }
    property Pen: TPen read FPen write FPen;		//     ,    
    property AxisX: TAxis read FAxisX write FAxisX;	//    
    property AxisY: TAxis read FAxisY write FAxisY;	//    
    property Grid: TGrid read FGrid write FGrid;	//    
    procedure AddPoint(X, Y: variant);			//        X,Y
    procedure PutMarker(X, Y: variant);			//        X,Y
    procedure Draw;					//  
    procedure Clear;					//    
    procedure InsertPoint(Index: integer; X, Y: variant); //      X,Y,   Index
    function PointCount: integer;			//    
    function MarkerCount: integer;			//    
    procedure DeletePoint(Index: integer);		//     Index
    procedure DeleteMarker(Index: integer);		//     Index
    constructor Create(lDiagram: TCustomDiagram);
    destructor Destroy; override;
    property ID: string read GetID write SetID;		//   ID
    property Point[Index: integer]: TVariantPoint read GetPoint write SetPoint; //      Index
    property Marker[Index: integer]: TVariantMarker read GetMarker write SetMarker; //      Index
    property MarkerColor: TColor read FMarkerColor write FMarkerColor; //    
  end;

type

  TDiagram = class(TCustomDiagram)
  private
    { Private declarations }
    FAOwner: TComponent;
    FCurveList: TList;
    FColorBackground: TColor;
    FOnReDraw: TDrawEvent;
    function GetColorBackground: TColor; override;
    procedure SetColorBackground(lColor: TColor); override;
    function GetCurveIx(Index: integer): TCurve;
    procedure SetCurveIx(Index: integer; Value: TCurve);
    function GetCurveCount: integer;
    function GetCurve(ID: string): TCurve;
    procedure SetCurve(ID: string; Value: TCurve);
  public
    { Public declarations }
    property Curve[ID: string]: TCurve read GetCurve write SetCurve;	//      ID
    property CurveIx[Index: integer]: TCurve read GetCurveIx write SetCurveIx; //       
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetIndex(ID: string): integer; 	//    ,  ID
    procedure AddCurve(ID: string); 		//   
    function DeleteCurve(ID: string): boolean;	//  
    procedure DeleteCurveIndex(Index: integer);	//     Index
    procedure ReDraw;				//   
    procedure Paint; override;
    property CurveCount: integer read GetCurveCount; //      
  published
    { Published declarations }
    property OnReDraw: TDrawEvent read FOnReDraw write FOnReDraw;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TDiagram]);
end;

constructor TAxis.Create(lCurve: TCustomCurve);
begin
  inherited Create;
  FCurve := lCurve;
  FPen := TPen.Create;
  FScalePen := TPen.Create;
  FFont := TFont.Create;
  Pen.Color := clBlack;
  Pen.Style := psSolid;
  Pen.Width := 2;
  Font.Color := clBlack;
  ScalePen.Color := clBlack;
  ScalePen.Style := psSolid;
  ScalePen.Width := 1;
  ArrowLeng := 20;
  ArrowWidth := 6;
  FFloatPrecision := 4;
  FFloatDigits := 1;
  DateTimeMode := [mHour, mMin];
end;

destructor TAxis.Destroy;
begin
  FPen.Destroy;
  FScalePen.Destroy;
  FFont.Destroy;
  inherited Destroy;
end;

constructor TGrid.Create(lCurve: TCustomCurve);
begin
  inherited Create;
  FCurve := lCurve;
  FGridStepX := 1;
  FGridStepY := 1;
  FGridStepX := (Curve.MaxX - Curve.MinX) / 10;
  FGridStepY := (Curve.MaxY - Curve.MinY) / 10;
  FPen := TPen.Create;
  Pen.Color := clGray;
  Pen.Style := psSolid;
  Pen.Width := 1;
end;

destructor TGrid.Destroy;
begin
  FPen.Destroy;
  inherited Destroy;
end;

constructor TCurve.Create(lDiagram: TCustomDiagram);
begin
  inherited Create;
  MinX := -12;
  MaxX := 12;
  MinY := -12;
  MaxY := 12;
  Visible := True;
  FDiagram := lDiagram;
  FPen := TPen.Create;
  FPointList := TList.Create;
  FMarkerList := TList.Create;
  FAxisX := TAxis.Create(Self);
  FAxisY := TAxis.Create(Self);
  FGrid := TGrid.Create(Self);
  Pen.Color := clBlack;
  Pen.Style := psSolid;
  Pen.Width := 1;
  Grid.Visible := False;
  Grid.GridStepX := 1;
  Grid.GridStepY := 1;
  AxisX.Visible := False;
  AxisY.Visible := False;
  AxisX.Coordinate := 0;
  AxisY.Coordinate := 0;
  AxisX.IndentMin := 5;
  AxisX.IndentMax := 5;
  AxisY.IndentMin := 5;
  AxisY.IndentMax := 5;
  AxisX.VisibleArrow := True;
  AxisY.VisibleArrow := True;
  AxisX.ArrowLeng := 20;
  AxisX.ArrowWidth := 6;
  AxisY.ArrowLeng := 20;
  AxisY.ArrowWidth := 6;
  AxisX.ScaleStep := 2;
  AxisY.ScaleStep := 2;
  AxisX.ScalePointReference := 0;
  AxisY.ScalePointReference := 0;
  AxisX.AxisType := atAbsciss;
  AxisY.AxisType := atOrdinate;
  AxisX.Distance := 5;
  AxisY.Distance := 5;
  AxisX.ScaleSize := 8;
  AxisY.ScaleSize := 8;
  AxisX.VisibleScale := True;
  AxisY.VisibleScale := True;
  AxisX.ScaleEmptyIntervalMin := -0.99;
  AxisY.ScaleEmptyIntervalMin := -0.99;
  AxisX.ScaleEmptyIntervalMax := 0.99;
  AxisY.ScaleEmptyIntervalMax := 0.99;
end;

destructor TCurve.Destroy;
begin
  while PointCount > 0 do
    DeletePoint(0);
  FPointList.Destroy;
  while MarkerCount > 0 do
    DeleteMarker(0);
  FMarkerList.Destroy;
  FPen.Destroy;
  FAxisY.Destroy;
  FAxisX.Destroy;
  FGrid.Destroy;
  inherited Destroy;
end;

constructor TDiagram.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAOwner := AOwner;
  FWidth := 0;
  FHeight := 0;
  FColorBackground := clWhite;
  FCurveList := TList.Create;
  AddCurve('Curve1');
  Curve['Curve1'].Grid.Visible := True;
  Curve['Curve1'].AxisX.Visible := True;
  Curve['Curve1'].AxisY.Visible := True;
end;

destructor TDiagram.Destroy;
var
  i: integer;
  FCurve: TCurve;
begin
  for i := 0 to FCurveList.Count - 1 do
  begin
    FCurve := FCurveList.Items[i];
    FCurve.Free;
  end;
  FCurveList.Destroy;
  inherited Destroy;
end;

procedure TCurve.SetMinX(Value: variant);
begin
  FMinX := Value;
end;

procedure TCurve.SetMinY(Value: variant);
begin
  FMinY := Value;
end;

procedure TCurve.SetMaxX(Value: variant);
begin
  FMaxX := Value;
end;

procedure TCurve.SetMaxY(Value: variant);
begin
  FMaxY := Value;
end;

procedure TCurve.AddPoint(X, Y: variant);
var
  X1, Y1, X2, Y2: variant;
  Width, Height: integer;
  PPoint: TPPoint;
  ix1, ix2, iy1, iy2: integer;
begin
  if assigned(Self) then
  begin
    Width := Diagram.Width;
    Height := Diagram.Height;
    if FPointList.Count > 0 then
    begin
      try
        X1 := (Width * (Point[PointCount - 1].X - FMinX)) / (FMaxX - FMinX);
        Y1 := Height - (Height * (Point[PointCount - 1].Y - FMinY)) / (FMaxY - FMinY);
        X2 := (Width * (X - FMinX)) / (FMaxX - FMinX);
        Y2 := Height - (Height * (Y - FMinY)) / (FMaxY - FMinY);
        ix1 := Round(X1);
        iy1 := Round(Y1);
        ix2 := Round(X2);
        iy2 := Round(Y2);
        if (ix1 <> ix2) or (iy1 <> iy2) then
        begin
          if Visible then
          begin
            Diagram.Canvas.Pen := Pen;
            Diagram.Canvas.MoveTo(ix1, iy1);
            Diagram.Canvas.LineTo(ix2, iy2);
          end;{if Visible - }
          New(PPoint);
          PPoint^.X := X;
          PPoint^.Y := Y;
          FPointList.Add(PPoint);
        end;{(ix1<>ix2) or (iy1<>iy2) -   
                                   - }
      except
        on EAbort do
          exit;
      end;{try except}
    end
    else
    begin
      New(PPoint);
      PPoint^.X := X;
      PPoint^.Y := Y;
      FPointList.Add(PPoint);
    end;{else}
  end;
end;

function TCurve.PointCount: integer;
begin
  Result := FPointList.Count;
end;

procedure TCurve.Clear;
var
  i: integer;
  PPoint: TPPoint;
begin
  for i := 0 to FPointList.Count - 1 do
  begin
    PPoint := FPointList.Items[i];
    Dispose(PPoint);
  end;
  FPointList.Clear;
end;

procedure TCurve.InsertPoint(Index: integer; X, Y: variant);
var
  PPoint: TPPoint;
begin
  if (FPointList.Count > 1) and (FPointList.Count > Index) then
  begin
    New(PPoint);
    PPoint^.X := X;
    PPoint^.Y := Y;
    FPointList.Insert(Index, PPoint);
  end;
end;

procedure TCurve.DeletePoint(Index: integer);
var
  PPoint: TPPoint;
begin
  PPoint := FPointList.Items[Index];
  Dispose(PPoint);
  FPointList.Delete(Index);
end;

procedure TCurve.SetPoint(Index: integer; Value: TVariantPoint);
begin
  FPointList.Items[Index] := @Value;
end;

function TCurve.GetPoint(Index: integer): TVariantPoint;
var
  PPoint: TPPoint;
begin
  PPoint := FPointList.Items[Index];
  Result := PPoint^;
end;

procedure TCurve.SetVisible(Value: boolean);
begin
  FVisible := Value;
end;

procedure TGrid.Draw;
var
  xc, yc: integer;
  x, y: variant;
begin
  if Visible then
  begin
    x := Curve.MinX;
    y := Curve.MinY;
    Curve.Diagram.Canvas.Pen := Pen;
    while x < Curve.MaxX do
    begin
      x := x + GridStepX;
      xc := round(Curve.Diagram.Width * (x - Curve.MinX) / (Curve.MaxX - Curve.MinX));
      Curve.Diagram.Canvas.MoveTo(xc, 0);
      Curve.Diagram.Canvas.LineTo(xc, Curve.Diagram.Height);
    end;{while}
    while y < Curve.MaxY do
    begin
      y := y + GridStepY;
      yc := round(Curve.Diagram.Height - (Curve.Diagram.Height * (y - Curve.MinY)) /
        (Curve.MaxY - Curve.MinY));
      Curve.Diagram.Canvas.MoveTo(0, yc);
      Curve.Diagram.Canvas.LineTo(Curve.Diagram.Width, yc);
    end;{while}
  end;{if}
end;

function TGrid.GetCurve: TCustomCurve;
begin
  Result := FCurve;
end;

procedure TGrid.SetCurve(lCurve: TCustomCurve);
begin
  FCurve := lCurve;
end;

procedure TGrid.SetGridStepX(Value: variant);
begin
  FGridStepX := Value;
end;

procedure TGrid.SetGridStepY(Value: variant);
begin
  FGridStepY := Value;
end;

procedure TGrid.SetVisible(Value: boolean);
begin
  FVisible := Value;
end;

procedure TAxis.SetCoordinate(Value: variant);
begin
  FCoordinate := Value;
end;

procedure TAxis.SetArrowWidth(Value: integer);
begin
  FArrowWidth := Value;
end;

procedure TAxis.SetArrowLeng(Value: integer);
begin
  FArrowLeng := Value;
end;

procedure TAxis.SetVisible(Value: boolean);
begin
  FVisible := Value;
end;

procedure TAxis.SetVisibleArrow(Value: boolean);
begin
  FVisibleArrow := Value;
end;

procedure TAxis.SetIndentMin(Value: integer);
begin
  FIndentMin := Value;
end;

procedure TAxis.SetIndentMax(Value: integer);
begin
  FIndentMax := Value;
end;

procedure TAxis.SetScaleStep(Value: variant);
begin
  FScaleStep := Value;
end;

procedure TAxis.SetScalePointReference(Value: variant);
begin
  FScalePointReference := Value;
end;

procedure TAxis.SetAxisType(Value: TAxisType);
begin
  FAxisType := Value;
end;

procedure TAxis.SetDistance(Value: integer);
begin
  FDistance := Value;
end;

procedure TAxis.SetVisibleScale(Value: boolean);
begin
  FVisibleScale := Value;
end;

procedure TAxis.SetScaleSize(Value: integer);
begin
  FScaleSize := Value;
end;

procedure TAxis.SetScaleEmptyIntervalMin(Value: variant);
begin
  FScaleEmptyIntervalMin := Value;
end;

procedure TAxis.SetScaleEmptyIntervalMax(Value: variant);
begin
  FScaleEmptyIntervalMax := Value;
end;

procedure TDiagram.ReDraw;
var
  i: integer;
begin
  Canvas.Brush.Color := FColorBackground;
  Canvas.Pen.Color := FColorBackground;
  Canvas.Rectangle(0, 0, Width, Height);
  for i := CurveCount - 1 downto 0 do
  begin
    CurveIx[i].Draw;
  end;{for}
end;

procedure TDiagram.SetColorBackground(lColor: TColor);
begin
  FColorBackground := lColor;
  if csDesigning in ComponentState then
    ReDraw;
end;

procedure TDiagram.Paint;
begin
  if (Self.Width <> FWidth) or (Self.Height <> FHeight) then
  begin
    FWidth := Self.Width;
    FHeight := Self.Height;
    Picture.Bitmap.Width := FWidth;
    Picture.Bitmap.Height := FHeight;
    ReDraw;
  end;
  inherited Paint;
end;

procedure TCurve.Draw;
var
  i, xc, yc: integer;
  x, y: variant;
begin
  if Visible then
  begin
    Grid.Draw;
    AxisX.Draw;
    AxisY.Draw;
    Diagram.Canvas.Pen := Pen;
    for i := 1 to PointCount - 1 do
    begin
      x := Point[i - 1].X;
      y := Point[i - 1].Y;
      xc := round((Diagram.Width * (x - MinX)) / (MaxX - MinX));
      yc := round(Diagram.Height - (Diagram.Height * (y - MinY)) / (MaxY - MinY));
      Diagram.Canvas.MoveTo(xc, yc);
      x := Point[i].X;
      y := Point[i].Y;
      xc := round((Diagram.Width * (x - MinX)) / (MaxX - MinX));
      yc := round(Diagram.Height - (Diagram.Height * (y - MinY)) / (MaxY - MinY));
      Diagram.Canvas.LineTo(xc, yc);
    end; {for}
    Diagram.Canvas.Pen.Color := FMarkerColor;
    for i := 0 to MarkerCount - 1 do
    begin
      x := Marker[i].X;
      y := Marker[i].Y;
      xc := round((Diagram.Width * (x - MinX)) / (MaxX - MinX));
      yc := round(Diagram.Height - (Diagram.Height * (y - MinY)) / (MaxY - MinY));
      Diagram.Canvas.Arc(xc - 3, yc - 3, xc + 3, yc + 3, 0, 0, 0, 0); //   
    end; {for}
  end; {if}
end;

function TCurve.GetDiagram: TCustomDiagram;
begin
  Result := FDiagram;
end;

procedure TCurve.SetDiagram(Value: TCustomDiagram);
begin
  if assigned(Self) then
    FDiagram := Value;
end;

function TCurve.GetMaxX: variant;
begin
  Result := FMaxX;
end;

function TCurve.GetMaxY: variant;
begin
  Result := FMaxY;
end;

function TCurve.GetMinX: variant;
begin
  Result := FMinX;
end;

function TCurve.GetMinY: variant;
begin
  Result := FMinY;
end;

function TCurve.GetVisible: boolean;
begin
  Result := FVisible;
end;

function TAxis.GetCurve: TCustomCurve;
begin
  Result := FCurve;
end;

procedure TAxis.SetCurve(lCurve: TCustomCurve);
begin
  FCurve := lCurve;
end;

procedure TAxis.Draw;
begin
  if AxisType = atAbsciss then
    DrawX;
  if AxisType = atOrdinate then
    DrawY;
end;

function TDiagram.GetColorBackground: TColor;
begin
  Result := FColorBackground;
end;

procedure TAxis.DrawX;
var
  xc, yc, correction: integer;
  i: byte;
  FStringList: TStringList;
  x: variant;
  st: string;
begin
  if Visible then
  begin
    Curve.Diagram.Canvas.Pen := Pen;
    yc := round(Curve.Diagram.Height - (Curve.Diagram.Height *
      (Coordinate - Curve.MinY)) / (Curve.MaxY - Curve.MinY));
    Curve.Diagram.Canvas.MoveTo(IndentMin, yc);
    Curve.Diagram.Canvas.LineTo(Curve.Diagram.Width - IndentMax, yc);
    if VisibleArrow then
    begin
      if ((Pen.Width shr 1) shl 1) = Pen.Width then
        correction := 1
      else
        correction := 0;
      Curve.Diagram.Canvas.Pen.Width := 1;
      Curve.Diagram.Canvas.Brush.Color := Pen.Color;
      Curve.Diagram.Canvas.Polygon([Point(Curve.Diagram.Width - IndentMax, yc),
        Point(Curve.Diagram.Width - IndentMax - ArrowLeng, yc -
        (ArrowWidth shr 1) - correction), Point(Curve.Diagram.Width -
        IndentMax - ArrowLeng, yc + (ArrowWidth shr 1)),
        Point(Curve.Diagram.Width - IndentMax, yc - correction)]);
    end;{if}
    if VisibleScale then
    begin
      Curve.Diagram.Canvas.Font := Font;
      yc := Curve.Diagram.Height - (Curve.Diagram.Height * (Coordinate - Curve.MinY)) /
        (Curve.MaxY - Curve.MinY);
      x := ScalePointReference;
      Curve.Diagram.Canvas.Pen := ScalePen;
      Curve.Diagram.Canvas.Brush.Color := Curve.Diagram.ColorBackground;
      while x < Curve.MaxX do
      begin
        xc := (Curve.Diagram.Width * (x - Curve.MinX)) / (Curve.MaxX - Curve.MinX);
        if (xc > IndentMin) and (xc < Curve.Diagram.Width - IndentMax) then
        begin
          Curve.Diagram.Canvas.MoveTo(xc, yc - (ScaleSize shr 1));
          Curve.Diagram.Canvas.LineTo(xc, yc + (ScaleSize shr 1));
          FStringList := ToStr(x);
          if (x < ScaleEmptyIntervalMin) or (x > ScaleEmptyIntervalMax) then
            for i := 0 to FStringList.Count - 1 do
            begin
              st := FStringList.Strings[i];
              Curve.Diagram.Canvas.TextOut(xc -
                (Curve.Diagram.Canvas.TextWidth(st) shr 1), yc +
                Distance + i * (Curve.Diagram.Canvas.TextHeight(st)), st);
            end;{for}
          FStringList.Free;
        end;{if}
        x := x + ScaleStep;
      end;{while}
      x := ScalePointReference;
      while x > Curve.MinX do
      begin
        x := x - ScaleStep;
        xc := (Curve.Diagram.Width * (x - Curve.MinX)) / (Curve.MaxX - Curve.MinX);
        if (xc > IndentMin) and (xc < Curve.Diagram.Width - IndentMax) then
        begin
          Curve.Diagram.Canvas.MoveTo(xc, yc - (ScaleSize shr 1));
          Curve.Diagram.Canvas.LineTo(xc, yc + (ScaleSize shr 1));
          FStringList := ToStr(x);
          if (x < ScaleEmptyIntervalMin) or (x > ScaleEmptyIntervalMax) then
            for i := 0 to FStringList.Count - 1 do
            begin
              st := FStringList.Strings[i];
              Curve.Diagram.Canvas.TextOut(xc -
                (Curve.Diagram.Canvas.TextWidth(st) shr 1), yc +
                Distance + i * (Curve.Diagram.Canvas.TextHeight(st)), st);
            end;{for}
          FStringList.Free;
        end;{if}
      end;{while}
    end;{if}
  end;{if}
end;

procedure TAxis.DrawY;
var
  xc, yc, correction: integer;
  y: variant;
  i: byte;
  st: string;
  FStringList: TStringList;
begin
  if Visible then
  begin
    Curve.Diagram.Canvas.Pen := Pen;
    xc := round((Curve.Diagram.Width * (Coordinate - Curve.MinX)) /
      (Curve.MaxX - Curve.MinX));
    Curve.Diagram.Canvas.MoveTo(xc, IndentMax);
    Curve.Diagram.Canvas.LineTo(xc, Curve.Diagram.Height - IndentMin);
    if VisibleArrow then
    begin
      if ((Pen.Width shr 1) shl 1) = Pen.Width then
        correction := 1
      else
        correction := 0;
      Curve.Diagram.Canvas.Pen.Width := 1;
      Curve.Diagram.Canvas.Brush.Color := Pen.Color;
      Curve.Diagram.Canvas.Polygon([Point(xc, IndentMax),
        Point(xc - correction - ArrowWidth shr 1, IndentMax + ArrowLeng),
        Point(xc + ArrowWidth shr 1, IndentMax + ArrowLeng),
        Point(xc - correction, IndentMax)]);
      if VisibleScale then
      begin
        xc := (Curve.Diagram.Width * (Coordinate - Curve.MinX)) /
          (Curve.MaxX - Curve.MinX);
        y := ScalePointReference;
        Curve.Diagram.Canvas.Pen := ScalePen;
        Curve.Diagram.Canvas.Brush.Color := Curve.Diagram.ColorBackground;
        while y < Curve.MaxY do
        begin
          yc := Curve.Diagram.Height - (Curve.Diagram.Height * (y - Curve.MinY)) /
            (Curve.MaxY - Curve.MinY);
          if (yc > IndentMin) and (yc < Curve.Diagram.Height - IndentMax) then
          begin
            Curve.Diagram.Canvas.MoveTo(xc - (ScaleSize shr 1), yc);
            Curve.Diagram.Canvas.LineTo(xc + (ScaleSize shr 1), yc);
            if (y < ScaleEmptyIntervalMin) or (y > ScaleEmptyIntervalMax) then
            begin
              FStringList := ToStr(y);
              st := '';
              for i := 0 to FStringList.Count - 1 do
                st := st + ' ' + FStringList.Strings[i];
              Delete(st, 1, 1);
              FStringList.Free;
              Curve.Diagram.Canvas.TextOut(xc - Distance -
                Curve.Diagram.Canvas.TextWidth(st), yc -
                (Curve.Diagram.Canvas.TextHeight(st) shr 1), st);
            end;
          end;{if}
          y := y + ScaleStep;
        end;{while}
        y := ScalePointReference;
        while y > Curve.MinY do
        begin
          y := y - ScaleStep;
          yc := Curve.Diagram.Height - (Curve.Diagram.Height * (y - Curve.MinY)) /
            (Curve.MaxY - Curve.MinY);
          if (yc > IndentMin) and (yc < Curve.Diagram.Height - IndentMax) then
          begin
            Curve.Diagram.Canvas.MoveTo(xc - (ScaleSize shr 1), yc);
            Curve.Diagram.Canvas.LineTo(xc + (ScaleSize shr 1), yc);
            if (y < ScaleEmptyIntervalMin) or (y > ScaleEmptyIntervalMax) then
            begin
              FStringList := ToStr(y);
              st := '';
              for i := 0 to FStringList.Count - 1 do
                st := st + ' ' + FStringList.Strings[i];
              Delete(st, 1, 1);
              FStringList.Free;
              Curve.Diagram.Canvas.TextOut(xc - Distance -
                Curve.Diagram.Canvas.TextWidth(st), yc -
                (Curve.Diagram.Canvas.TextHeight(st) shr 1), st);
            end;
          end;{if}
        end;{while}
      end;{if}
    end;{if}
  end;{if}
end;


function TCurve.GetID: string;
begin
  Result := FID;
end;

procedure TCurve.SetID(Value: string);
begin
  FID := Value;
end;

function TDiagram.GetCurveIx(Index: integer): TCurve;
var
  FCurve: TCurve;
begin
  FCurve := FCurveList.Items[Index];
  Result := FCurve;
end;

procedure TDiagram.SetCurveIx(Index: integer; Value: TCurve);
begin
  FCurveList.Items[Index] := Value;
end;

procedure TDiagram.AddCurve(ID: string);
var
  FCurve: TCurve;
begin
  FCurve := TCurve.Create(Self);
  FCurve.ID := ID;
  FCurveList.Add(FCurve);
end;

function TDiagram.DeleteCurve(ID: string): boolean;
var
  FCurve: TCurve;
  ix: integer;
begin
  Result := False;
  ix := GetIndex(ID);
  if ix = -1 then
    exit
  else
  begin
    FCurve := CurveIx[ix];
    FCurve.Free;
    FCurveList.Delete(ix);
  end;{else}
end;

procedure TDiagram.DeleteCurveIndex(Index: integer);
var
  FCurve: TCurve;
begin
  FCurve := FCurveList.Items[index];
  FCurve.Free;
  FCurveList.Delete(index);
end;

function TDiagram.GetIndex(ID: string): integer;
var
  FCurve: TCurve;
  i, Count: integer;
begin
  i := 0;
  Result := -1;
  Count := FCurveList.Count;
  repeat
    FCurve := FCurveList.Items[i];
    if FCurve.ID = ID then
      Result := i;
    Inc(i);
  until i >= Count;
end;

function TDiagram.GetCurveCount: integer;
begin
  Result := FCurveList.Count;
end;

function TDiagram.GetCurve(ID: string): TCurve;
var
  ix: integer;
  FCurve: TCurve;
begin
  ix := GetIndex(ID);
  if ix = -1 then
    Result := nil
  else
    FCurve := FCurveList.Items[ix];
  Result := FCurve;
end;

procedure TDiagram.SetCurve(ID: string; Value: TCurve);
var
  ix: integer;
begin
  ix := GetIndex(ID);
  if ix <> -1 then
    FCurveList.Items[ix] := Value;
end;

function TAxis.ToStr(Value: variant): TStringList;
var
  FStringList: TStringList;
  st: string;
  fNum: double;
  wYear, wMonth, wDay, wHour, wMinute, wSecond, wMilliSecond: word;

  function CheckLeng(s: string; Leng: byte): string;
  begin
    if Leng = 2 then
      if length(s) = 1 then
        Result := '0' + s
      else
        Result := s;
    if Leng = 3 then
      if length(s) = 1 then
        Result := '00' + s
      else
      if length(s) = 2 then
        Result := '00' + s
      else
        Result := s;
  end;

begin
  FStringList := TStringList.Create;
  st := '';
  case VarType(Value) of
    varInteger: FStringList.Add(IntToStr(Value));
    varSmallint: FStringList.Add(IntToStr(Value));
    varByte: FStringList.Add(IntToStr(Value));
    varSingle:
    begin
      fNum := Value;
      FStringList.Add(FloatToStrF(fNum, ffFixed, FFloatPrecision, FFloatDigits));
    end;
    varDouble:
    begin
      fNum := Value;
      FStringList.Add(FloatToStrF(fNum, ffFixed, FFloatPrecision, FFloatDigits));
    end;
    varCurrency:
    begin
      fNum := Value;
      FStringList.Add(FloatToStrF(fNum, ffFixed, FFloatPrecision, FFloatDigits));
    end;
    varBoolean: if Value then
        FStringList.Add('true')
      else
        FStringList.Add('false');
    varString: FStringList.Add(Value);
    varDate:
    begin
      DecodeDate(Value, wYear, wMonth, wDay);
      if mDay in DateTimeMode then
        st := st + CheckLeng(IntToStr(wDay), 2) + '.';
      if mMonth in DateTimeMode then
        st := st + CheckLeng(IntToStr(wMonth), 2) + '.';
      if mYear in DateTimeMode then
        st := st + IntToStr(wYear);
      if st <> '' then
        if st[length(st)] = '.' then
          Delete(st, length(st), 1);
      if st <> '' then
        FStringList.Add(st);
      st := '';
      DecodeTime(Value, wHour, wMinute, wSecond, wMilliSecond);
      if mHour in DateTimeMode then
        st := st + CheckLeng(IntToStr(wHour), 2) + ':';
      if mMin in DateTimeMode then
        st := st + CheckLeng(IntToStr(wMinute), 2) + ':';
      if mSec in DateTimeMode then
        st := st + CheckLeng(IntToStr(wSecond), 2);
      if st <> '' then
        if st[length(st)] = ':' then
          Delete(st, length(st), 1);
      if mMSec in DateTimeMode then
      begin
        if st <> '' then
          st := st + '.';
        st := st + CheckLeng(IntToStr(wMilliSecond), 3);
      end;
      if st <> '' then
        FStringList.Add(st);
    end {varDate}
    else {case VarType}
  end; {case VarType}
  Result := FStringList;
end;

procedure TCurve.PutMarker(X, Y: variant);
var
  MX, MY: variant;
  Width, Height: integer;
  PMarker: TPMarker;
begin
  Width := Diagram.Width;
  Height := Diagram.Height;
  MX := (Width * (X - FMinX)) / (FMaxX - FMinX);
  MY := Height - (Height * (Y - FMinY)) / (FMaxY - FMinY);
  Diagram.Canvas.Pen := Pen;
  Diagram.Canvas.Arc(MX - 2, MY - 2, MX + 2, MY + 2, 0, 0, 0, 0); //   
  New(PMarker);
  PMarker^.X := X;
  PMarker^.Y := Y;
  FMarkerList.Add(PMarker);
end;

procedure TCurve.DeleteMarker(Index: integer);
var
  PMarker: TPMarker;
begin
  PMarker := FMarkerList.Items[Index];
  Dispose(PMarker);
  FMarkerList.Delete(Index);
end;

function TCurve.MarkerCount: integer;
begin
  Result := FMarkerList.Count;
end;

function TCurve.GetMarker(Index: integer): TVariantMarker;
var
  PMarker: TPMarker;
begin
  PMarker := FMarkerList.Items[Index];
  Result := PMarker^;
end;

procedure TCurve.SetMarker(Index: integer; Value: TVariantMarker);
begin
  FMarkerList.Items[Index] := @Value;
end;

end.